/**
 * A simple fault injector and a simple Replication Manager/Factory
 * for our project that we can use in testing our fault-tolerant
 * baseline implementations on the ECE Linux Cluster.
 * 
 * Usage:
 * 	(for launching a Jboss process)
 * 		repman -p <program and args> -l <label> [-m <machine>]
 *  (for listing live processes
 *      repman
 *  (for killing live processes)
 *     repman -k <label>
 *
 *
 * History:
 *
 * Date          Author              Change
 * ----------    ----------------    ---------------------------------------------------------------
 * 02/17/2004    GEO		         Initial version.
*/

import java.io.*;
import java.util.StringTokenizer;

public class repman
{
	// launch a remote or local Jboss process
	public static int launch(String machine, String realCmd, String label)
    {
        String cmd = null;
        int exitCode = 0;
        
        try
        {
        		cmd = SSH_PATH + machine + realCmd + " 2>&1 &";		
                System.out.println("Launching: "+ cmd);
                Process p = Runtime.getRuntime().exec(cmd);

                try
                {
                    // wait for exit code -- if it's 0, command worked,
                    exitCode = p.waitFor();
                    if (exitCode==0)
                    {
                    	System.out.println("Server("+label+") Launched Successfully");
                    	writeLog(machine, label);
                    }
       				else System.out.println("Fail to launch a server!! Retry");                   
                }
                catch(InterruptedException x)
                {
                    System.err.println("Error launching cmd='" + cmd + "'");
                    System.err.println("Caught exception: " + x);
                }
                
        }
        catch(IOException x)
        {
            System.err.println("Could not launch cmd=" + cmd);
            System.err.println("Caught exception: " + x);
        }
	return exitCode;
    }
    
    // display error message and usage when user input is wrong
	public static void syntaxError() {
    	System.out.println("Wrong syntax");
    	System.out.println("Usage: (for launching a Jboss process)  repman -p <program and args> -l <label> [-m <machine>]");
    	System.out.println("       (for listing live processes)     repman");
    	System.out.println("       (for killing live processes)     repman -k <label>");
    }

    // add the launched Jboss process to the list file
	public static void writeLog(String machine, String label)
    {
		FileWriter out = null;
	
		try
		{
			out = new FileWriter("process.txt",true);
        	out.write(label+" "+machine+"\n");
        	out.flush();
		}catch(IOException x)
		{
			x.printStackTrace();
		}
    }

	// get local server name if no machine is designated
    public static String getServerName() throws IOException
	{
    	Process p = Runtime.getRuntime().exec("hostname");
    	
    	BufferedReader input =
    		new BufferedReader
			(new InputStreamReader(p.getInputStream()));
    	
    	return input.readLine();
    }
	
    // check for machine name and label name with already launched process list file
    // duplicated name is not allowed
    public static boolean checklabel(String machine, String label) throws IOException
    {
     	BufferedReader in = new BufferedReader(new FileReader("process.txt"));
    	
    	String line = null;
    	String sLabel = null;
    	String sMachine = null;
    	
    	while (in.ready() ){
    		
    		line = in.readLine();
    		StringTokenizer objTokenizer = new StringTokenizer(line);

    		// get the process's Label and Machine.
    		sLabel = objTokenizer.nextToken();
    		sMachine = objTokenizer.nextToken();
    		if (sLabel.compareTo(label)==0)
    		{
    			return false;
    		}
    		else if (sMachine.compareTo(machine)==0)
    		{	
    			return false;
    		}
    		else
    			;
    	}
		return true;
    }

    // delete the killed server from the launched server list file
    public static String deleteLog(String label) throws IOException
	{
    	BufferedReader fromFile = new BufferedReader(new FileReader("process.txt"));
    	FileWriter toTemp = new FileWriter("temp");
    	
    	String line = "";
    	String sLabel = "";
    	String sMachine = "";
    	String sTarget = "";
    	int i=0;
    	while (fromFile.ready())
    	{
    		String temp = fromFile.readLine();
    		toTemp.write(temp+"\n");
    		toTemp.flush();
    		i++;
    	}
    	fromFile.close();
    	toTemp.close();
  
    	BufferedReader fromTemp = new BufferedReader(new FileReader("temp"));
    	FileWriter toFile = new FileWriter("process.txt");
    		
    	while (fromTemp.ready()){
    		
    		line = fromTemp.readLine();
    		StringTokenizer objTokenizer = new StringTokenizer(line);
    		
    		// get the process's label and machine.
    		sLabel = (String)objTokenizer.nextToken();
    		sMachine = objTokenizer.nextToken();
		label = (String)label;

    		if (sLabel.trim().equals(label.trim()))
		{
    			sTarget = sMachine;
}
    		else
    		{
    			toFile.write(line+"\n");
    			toFile.flush() ;
    		}
    			
    	}
    	
    	toFile.close();
    	fromTemp.close();
    	return sTarget;

    }
    
    public static void main(String[] args)
    {
    	int checkRun = 0;	        // check for whether both program and label swithces are used 
    	int checkMachine = 0;       // check for whether machine switch is used or not
    	int checkKill = 0;          // check for whether kill switch is used or not
    	int checkOption = 0;		//1: launch without local machine
    								//2: launch with specified machine
    								//3: kill server
    								//4: syntax Error;
    								//5: duplicated label;
    	
    	String program = null;
    	String labelRun = null;     // label for the launched process
    	String labelKill = null;    // label for the process to be killed
    	String machine = null;
    	
    	BufferedReader in = null;
    	String line = null;
    	
    	try {

    		// check for the number of input arguments
    		// the number of input arguments should be even number
    		if (args.length%2 == 0) {
    			
    			// list launched Jboss servers when there is no input argument
    			if (args.length == 0) {
    				in = new BufferedReader(new FileReader("process.txt"));
    				while ((line = in.readLine()) != null) {
    					System.out.println(line);
    				}
    			}
    			
    			else {
    				int i;
    				
    				// check for valid switchs
    				
    				// '-p'(program) and '-l'(label) switches are mandatory
    				// and '-m'(machine) switch is optional
    				
    				// or '-k'(kill) switch only can be used
    				for (i=0;i<args.length;i=i+2) {
    					if (args[i].equals("-p")) {
    						program = args[i+1];
    						checkRun++;
    					}
    					else if (args[i].equals("-l")) {
    						labelRun = args[i+1];
    						checkRun++;
    					}
    					else if (args[i].equals("-m")) {
    						machine = args[i+1];
    						checkMachine++;
    					}
    					else if (args[i].equals("-k")) {
    						labelKill = args[i+1];
    						checkKill++;
    						break;
    					}
    				}

    				// program and label switches are used
    				if (checkRun == 2 && checkKill == 0) {
    					
    					// machine name is local server name when it is not designated
    					if (checkMachine==0) machine = getServerName();
    					System.out.println("local:"+machine);
    					
    					// check for whether program and label are valid
    					// they should not be duplicated with already launched processes
    					boolean cFlag = checklabel(machine,labelRun);
    					
    					if (!cFlag)
    					{
    						checkOption=5;	//Duplicated machine or Label;
    					}
    					else{
    						if (checkMachine == 1)
    							checkOption= 2;		//Launch remote machine
    						else
    							checkOption = 1;	//Launch local machine
    					}
    				}
    				
    				// kill switch is used 
    				else if (checkRun == 0 && checkKill == 1) {
    					checkOption=3;	//Kill Server
    				}
    				
    				// display error when the switches are used wrongly
    				else {
    					checkOption=4;	//Syntax Error
    				}
    				int result;
    				switch (checkOption)
					{
					case 1: //launch local server
						result = launch("", " /afs/ece.cmu.edu/class/ece846/java/jboss-3.2.3/bin/"
								+ program + " -c team1" + " >" + CWD + "/" + labelRun + ".txt", labelRun);
						break;
					case 2: //launch remote server
						result = launch(machine, " /afs/ece.cmu.edu/class/ece846/java/jboss-3.2.3/bin/"
								+ program + " -c team1" + " >" + CWD + "/" + labelRun + ".txt", labelRun);
						break;
					case 3: //kill server
						String killed = deleteLog(labelKill); // get machine name to be killed
						System.out.println("killed: " + killed);
						String cmd = SSH_PATH + killed + " /afs/ece/usr/younbokl/analysis/kill.sh";
						Runtime.getRuntime().exec(cmd);
						break;
					case 4: //wrong syntax with switches
						syntaxError();
						break;
					case 5:
						System.out.println("Duplicated machine or label!! Please retry!!");
    				}
    			}    	
    		}
    		
    		// display error when the number of arguments is wrong
    		else {
    			syntaxError();
    		}

    	} catch (IOException e) {
    		e.printStackTrace();	
    	}
    }
    
    // The absolute pathname of ssh.
    private static final String SSH_PATH = "/usr/local/bin/ssh ";
    //private static final String JBOSS_HOME = "/"

    // Current working directory
    private static String CWD = System.getProperty("user.dir", "");
}
